package pl.nettigo.esp.configurator;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.InputType;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

import java.lang.reflect.Type;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.SocketException;

import pl.nettigo.esp.configurator.adapter.NetworksAdapter;
import pl.nettigo.esp.configurator.exception.UnableToStartUdpServerException;
import pl.nettigo.esp.configurator.model.BaseResponse;
import pl.nettigo.esp.configurator.model.Command;
import pl.nettigo.esp.configurator.model.Response;
import pl.nettigo.esp.configurator.model.WirelessCredentials;
import pl.nettigo.esp.configurator.model.WirelessNetwork;
import pl.nettigo.esp.configurator.util.UdpMessenger;

public class NetworkSetupActivity extends AppCompatActivity implements NetworksAdapter.OnWirelessNetworkItemClickListener, UdpMessenger.OnPacketReceiveListener {

    private RecyclerView mNetworksRecycler;
    private NetworksAdapter mAdapter = new NetworksAdapter(this);
    private UdpMessenger mUdpMessenger;
    private InetAddress mIpAddress;
    private Button mScanNetworksButton;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_network_setup);
        ActionBar actionBar = getSupportActionBar();
        if(actionBar != null) {
            actionBar.setDisplayHomeAsUpEnabled(true);
        }

        try {
            mUdpMessenger = UdpMessenger.getInstance();
        } catch (SocketException e) {
            throw new UnableToStartUdpServerException();
        }

        Intent intent = getIntent();

        mIpAddress = (InetAddress) intent.getSerializableExtra(MainActivity.EXTRA_IP_ADDRESS);

        if(mIpAddress == null) {
            throw new RuntimeException("Do not run this activity without proper intent!");
        }

        mNetworksRecycler = findViewById(R.id.networks_list);
        mScanNetworksButton = findViewById(R.id.networks_scan);

        mNetworksRecycler.setHasFixedSize(false);

        RecyclerView.LayoutManager layoutManager = new LinearLayoutManager(this);
        mNetworksRecycler.setLayoutManager(layoutManager);

        mAdapter.setOnWirelessNetworkItemClickListener(this);
        mNetworksRecycler.setAdapter(mAdapter);

        mScanNetworksButton.setOnClickListener(new Button.OnClickListener() {
            @Override
            public void onClick(View view) {
                onScanNetworksButtonClick();
            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
        mUdpMessenger.setOnPacketReceiveListener(this);
        mUdpMessenger.setTargetAddress(mIpAddress);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                finish();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    public void onWirelessNetworkItemClick(final WirelessNetwork network) {
        if(network.isProtected()) {
            final EditText passwordEditText = new EditText(this);
            passwordEditText.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_PASSWORD);

            AlertDialog dialog = new AlertDialog.Builder(this)
                    .setTitle(network.getSSID())
                    .setMessage(R.string.label_type_network_password)
                    .setView(passwordEditText)
                    .setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            String password = passwordEditText.getText().toString();
                            WirelessCredentials credentials = new WirelessCredentials(network.getSSID(), password);
                            Command<WirelessCredentials> queryData = new Command<>("setWiFi", credentials);
                            String data = UdpMessenger.buildJsonString(queryData);
                            mUdpMessenger.sendData(data);
                        }
                    })
                    .setNegativeButton(R.string.action_cancel, null)
                    .create();
            dialog.show();
        } else {
            WirelessCredentials credentials = new WirelessCredentials(network.getSSID());
            Command<WirelessCredentials> command = new Command<>("setWiFi", credentials);
            String data = UdpMessenger.buildJsonString(command);
            mUdpMessenger.sendData(data);
        }

    }

    public void onScanNetworksButtonClick() {
        Command command = new Command("fetchNetworks");
        mAdapter.clearList();
        String data = UdpMessenger.buildJsonString(command);
        mUdpMessenger.sendData(data);
    }

    @Override
    public void onPacketReceive(DatagramPacket packet) {
        String json = UdpMessenger.parseDatagramPacket(packet);
        BaseResponse data = new Gson().fromJson(json, BaseResponse.class);
        if(data.getType().equals("networkData")) {
            Type responseType = new TypeToken<Response<WirelessNetwork>>(){}.getType();
            final Response<WirelessNetwork> targetObject = new Gson().fromJson(json, responseType);
            runOnUiThread(new Runnable() {
                public void run() {
                    mAdapter.addNetwork(targetObject.getResult());
                }
            });
        } else if(data.getType().equals("setWiFiFeedback")) {
            Type responseType = new TypeToken<Response<String>>(){}.getType();
            final Response<String> targetObject = new Gson().fromJson(json, responseType);
            runOnUiThread(new Runnable() {
                public void run() {
                    Toast.makeText(NetworkSetupActivity.this, targetObject.getResult(), Toast.LENGTH_SHORT).show();
                }
            });
        }
    }
}
